
#Created on 12 Aug 2014

#@author: neil.butcher


import sqlite3
from .MeasurementDatabase import filename
from .CurrentUnitSetter import setter
from PyQt4 import QtCore




class UnitMeasurementException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class Measurement(object):
    def __init__(self, name=None):
        self._name = name
        self._unitsCache = None
        self._id_cache = None
        self._baseUnitCache = None

    def __repr__(self):
        return "Measurement('" + self._name + "')"

    @property
    def baseUnit(self):
        """
        :rtype: Unit
        """
        if self._unitsCache is None:
            self._units()
        return self._baseUnitCache

    @property
    def name(self):
        """
        :rtype: str
        """
        _connection = sqlite3.connect(filename, detect_types=sqlite3.PARSE_DECLTYPES)
        cursor = _connection.execute("SELECT name  FROM MEASUREMENTNAMES WHERE measurementID = ? AND preferred = 1 ",
                                     (self._id(),))
        for row in cursor:
            return row[0]

        _connection.execute("UPDATE MEASUREMENTNAMES set preferred = 1 where measurementID = ? AND name = ? ",
                                (self._id_cache, self._name))
        _connection.commit()
        return self._name


    def setPreferredName(self, name):
        """
        :type name: str
        """
        if name != self.name:
            _connection = sqlite3.connect(filename, detect_types=sqlite3.PARSE_DECLTYPES)
            _connection.execute("UPDATE MEASUREMENTNAMES set preferred = 0 where measurementID = ?",
                                (self._id_cache,))
            _connection.execute("UPDATE MEASUREMENTNAMES set preferred = 1 where measurementID = ? AND name = ? ",
                                (self._id_cache, name))
            _connection.commit()


    def addAlias(self, name):
        """
        :type name: str
        """
        if name != self.name:
            _connection = sqlite3.connect(filename, detect_types=sqlite3.PARSE_DECLTYPES)
            _connection.execute("INSERT INTO MEASUREMENTNAMES VALUES (?,?,?) ",
                                (self._id_cache, name, 0))
            _connection.commit()

    def _units(self):
        if self._unitsCache is None:
            self._unitsCache = {}
            _connection = sqlite3.connect(filename, detect_types=sqlite3.PARSE_DECLTYPES)
            cursor = _connection.execute("SELECT name , Scale , offset ,id , base FROM UNITS WHERE measurementID = ?",
                                         (self._id(),))
            for row in cursor:
                unit = Unit()
                unit.measurement = self
                unit._name = row[0]
                unit.scale = float(row[1])
                unit.offset = float(row[2])
                unit.id_cache = row[3]
                self._unitsCache[row[3]] = unit
                if row[4] == 1:
                    self._baseUnitCache = unit
            if self._baseUnitCache is None:
                raise UnitMeasurementException("There was no unit to act as the base unit for measurement " + self._name)
        return self._unitsCache

    @property
    def units(self):
        return list(self._units().values())

    def _id(self):
        if self._id_cache is None:
            _connection = sqlite3.connect(filename, detect_types=sqlite3.PARSE_DECLTYPES)
            cursor = _connection.execute("SELECT id  FROM MEASUREMENTS WHERE name = ?", (self._name,))
            for row in cursor:
                if self._id_cache is None:
                    self._id_cache = row[0]
                else:
                    raise UnitMeasurementException("There are multiple measurements with the same name")
            if self._id_cache is None:
                cursor = _connection.execute("SELECT measurementID  FROM MEASUREMENTNAMES WHERE name = ?", (self._name,))
                for row in cursor:
                    if self._id_cache is None:
                        self._id_cache = row[0]
                    else:
                        raise UnitMeasurementException("There are multiple measurements with the same name")
                if self._id_cache is None:
                    raise UnitMeasurementException("There was no measurements with this name in the database")
        return self._id_cache

    def currentUnit(self, label='normal'):
        """
        :type label: str
        :rtype: Unit
        """
        _connection = sqlite3.connect(filename, detect_types=sqlite3.PARSE_DECLTYPES)
        cursor = _connection.execute("SELECT unitID  FROM CURRENTUNITS WHERE measurementID = ? AND label = ? ",
                                     (self._id(), label))
        for row in cursor:
            return self._units()[row[0]]

        self._units()
        _connection.execute("INSERT INTO CURRENTUNITS VALUES (?,?,?) ",
                            (self._id_cache, label, self.baseUnit.id_cache))
        _connection.commit()
        return self.baseUnit

    def setCurrentUnit(self, u, label='normal'):
        """
        :type u: Unit
        :type label: str
        """
        if u != self.currentUnit(label):
            _connection = sqlite3.connect(filename, detect_types=sqlite3.PARSE_DECLTYPES)
            _connection.execute("UPDATE CURRENTUNITS set unitID = ? where measurementID = ? AND label = ? ",
                                (u.id_cache, self._id_cache, label))
            _connection.commit()

    def report(self, base_value, precision=6, label='normal', writeUnit=True):
        try:
            scaled_value = self.currentUnit(label).scaledValueOf(base_value)
            text = QtCore.QLocale().toString(scaled_value, precision=precision)
        except (ValueError, TypeError):
            text = str(base_value)
        if writeUnit:
            text = text  + ' (' + self.currentUnit(label).name + ')'
        return text

    def scaledValueOf(self, base_float, label='normal'):
        return self.currentUnit(label).scaledValueOf(base_float)

    def baseValueFrom(self, base_float, label='normal'):
        return self.currentUnit(label).baseValueFrom(base_float)


class Unit(object):
    def __init__(self):
        self._name = None
        self.measurement = None
        self.scale = 1.0
        self.offset = 0.0
        self.id_cache = 0

    def __repr__(self):
        return "Unit(" +str(self.measurement) + ",'" + self.name + "')"

    def scaledValueOf(self, base_float):
        return (base_float / self.scale ) - self.offset

    def baseValueFrom(self, scaled_float):
        return (scaled_float + self.offset) * self.scale

    def scaledDeltaValueOf(self, base_float):
        # scale a change in the measurement (rather than an absolute value)
        #eg a change of 1Kelvin = a change of 1degC
        return (base_float / self.scale )

    def baseDeltaValueFrom(self, scaled_float):
        # scale a change in the measurement (rather than an absolute value)
        #eg a change of 1Kelvin = a change of 1degC
        return scaled_float * self.scale

    @property
    def baseUnit(self):
        """
        :rtype: Unit
        """
        return self.measurement.baseUnit

    def currentUnit(self, label='normal'):
        """
        :type label: str
        :rtype: Unit
        """
        return self.measurement.currentUnit(label=label)

    def becomeCurrentNormalUnit(self):
        setter.setMeasurementUnit(self.measurement, self)
    
    def alias(self):
        res = []
        _connection = sqlite3.connect(filename, detect_types=sqlite3.PARSE_DECLTYPES)
        cursor = _connection.execute("SELECT name  FROM UNITNAMES WHERE unitID = ?",
                                     (self.id_cache,))
        for row in cursor:
            res.append(row[0])
        return res

    def addAlias(self, name):
        """
        :type name: str
        """
        if name != self.name:
            _connection = sqlite3.connect(filename, detect_types=sqlite3.PARSE_DECLTYPES)
            _connection.execute("INSERT INTO UNITNAMES VALUES (?,?,?) ",
                                (self.id_cache, name, 0))
            _connection.commit()

    @property
    def name(self):
        """
        :rtype: str
        """
        _connection = sqlite3.connect(filename, detect_types=sqlite3.PARSE_DECLTYPES)
        cursor = _connection.execute("SELECT name  FROM UNITNAMES WHERE unitID = ? AND preferred = ? ",
                                     (self.id_cache, 1))
        for row in cursor:
            return row[0]

        _connection.execute("UPDATE UNITNAMES set preferred = ? where unitID = ? AND name = ? ",
                                (1, self.id_cache, self._name))
        _connection.commit()
        return self._name


    def setPreferredName(self, name):
        """
        :type name: str
        """
        if name != self.name:
            _connection = sqlite3.connect(filename, detect_types=sqlite3.PARSE_DECLTYPES)
            _connection.execute("UPDATE UNITNAMES set preferred = ? where unitID = ?",
                                (0, self.id_cache,))
            _connection.execute("UPDATE UNITNAMES set preferred = ? where unitID = ? AND name = ? ",
                                (1, self.id_cache, name))
            _connection.commit()



#Created on 17 Aug 2014

#@author: neil.butcher


import sys

from . import MeasurementDatabase
from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import pyqtSignal

from .MeasurementWidgets import UnitDisplay, UnitComboBox, UnitSpinBox


class UnitEntryField(QtGui.QWidget):
    valueChanged = pyqtSignal(float)

    def __init__(self, parent, measurement=None, measurementLabel='normal', label=None, delta=False, editableUnit=True):
        QtGui.QWidget.__init__(self, parent)
        self.layout = QtGui.QHBoxLayout(self)
        self.layout.setMargin(0)
        if not label == None:
            a = QtGui.QLabel(label, parent=self)
            self.layout.addWidget(a)
        self._box = UnitSpinBox(self, measurement, delta=delta, measurementLabel=measurementLabel)
        self._box.valueChanged.connect(self._valueChanged)
        self.layout.addWidget(self._box)
        self._measurement = measurement
        if editableUnit:
            self._unitfield = UnitComboBox(self, measurement, measurementLabel=measurementLabel)
        else:
            self._unitfield = UnitDisplay(self, measurement, measurementLabel=measurementLabel)
        self.layout.addWidget(self._unitfield)

    def _valueChanged(self, f):
        self.valueChanged.emit(f)

    def setMeasurement(self, measurement):
        self._measurement = measurement
        self._box.setMeasurement(measurement)
        self._unitfield.setMeasurement(measurement)

    def setMeasurementName(self, string):
        self.setMeasurement(MeasurementDatabase.Measurement(string))

    def value(self):
        return self._box.value()

    def setValue(self, f):
        return self._box.setValue(f)

    def setReadOnly(self, a_bool):
        self._box.setEnabled(not a_bool)

class SingleMeasurementEntryFieldStack(QtGui.QWidget):
    valueChanged = pyqtSignal(float, str)

    def __init__(self, parent, n=1, measurement=None, measurementLabel='normal', labels=None, deltas=None):
        QtGui.QWidget.__init__(self, parent)
        self.layout = QtGui.QVBoxLayout(self)
        self.layout.setSpacing(0)
        self.layout.setMargin(0)
        signalMapper = QtCore.QSignalMapper(self)

        self.fields = [0] * n
        if labels is None:
            l = [None for _ in range(n)]
            self.identity = [str(i + 1) for i in range(n)]
        else:
            l = [str(i) for i in labels]
            self.identity = [str(i) for i in labels]
        if deltas is None:
            d = [None for _ in range(n)]
        else:
            d = deltas
        e = [False for _ in range(n)]
        e[0] = True

        for i in range(n):
            aField = UnitEntryField(self, measurement=measurement, measurementLabel=measurementLabel, label=l[i],
                                    delta=d[i], editableUnit=e[i])
            self.fields[i] = aField
            self.layout.addWidget(aField)
            aField.valueChanged.connect(signalMapper.map);
            signalMapper.setMapping(aField, i);

        signalMapper.mapped.connect(self._valueChanged)


    def setMeasurement(self, measurement):
        self._measurement = measurement
        for f in self.fields:
            f.setMeasurement(measurement)

    def _valueChanged(self, i):
        value = self.fields[i].value()
        identity = self.identity[i]
        print (value, identity)
        self.valueChanged.emit(value, identity)


class MeasurementEntryGridField(QtGui.QWidget):
    valueChanged = pyqtSignal(float, str)
    editingFinished = pyqtSignal(float, str)

    def __init__(self, parent):
        QtGui.QWidget.__init__(self, parent)
        self.layout = QtGui.QGridLayout(self)
        self.layout.setMargin(2)
        self.layout.setSpacing(6)
        self.valueSignalMapper = QtCore.QSignalMapper(self)
        self.valueSignalMapper.mapped.connect(self._valueChanged)
        self.editingSignalMapper = QtCore.QSignalMapper(self)
        self.editingSignalMapper.mapped.connect(self._editingFinished)
        self.identifers = []
        self.boxes = []
        self.measurements = []

    def addField(self, measurement, identifer, measurementLabel='normal', label=None, delta=False, editable=None):
        nextIndex = len(self.identifers)
        if editable is None:
            _editable = (measurement, measurementLabel) not in self.measurements
        else:
            _editable = editable

        if not label is None:
            a = QtGui.QLabel(label, parent=self)
            a.setMargin(0)
            self.layout.addWidget(a, nextIndex, 0)

        _box = UnitSpinBox(self, measurement, measurementLabel=measurementLabel, delta=delta)
        _box.setMargin(0)
        self.layout.addWidget(_box, nextIndex, 1)
        _box.valueChanged.connect(self.valueSignalMapper.map)
        self.valueSignalMapper.setMapping(_box, nextIndex)
        _box.editingFinished.connect(self.editingSignalMapper.map)
        self.editingSignalMapper.setMapping(_box, nextIndex)
        self.boxes.append(_box)

        if _editable and measurement is not None:
            self._unitfield = UnitComboBox(self, measurement, measurementLabel=measurementLabel)
        else:
            self._unitfield = UnitDisplay(self, measurement, measurementLabel=measurementLabel)
        self._unitfield.setMargin(0)
        self.layout.addWidget(self._unitfield, nextIndex, 2)

        self.identifers.append(identifer)
        self.measurements.append((measurement, measurementLabel))


    def _valueChanged(self, i):
        value = self.boxes[i].value()
        identity = self.identifers[i]
        self.valueChanged.emit(value, identity)

    def _editingFinished(self, i):
        value = self.boxes[i].value()
        identity = self.identifers[i]
        self.editingFinished.emit(value, identity)

    def setValue(self, identity, value):
        i = self.identifers.index(identity, )
        value = self.boxes[i].setValue(value)

    def value(self, identity):
        i = self.identifers.index(identity, )
        return self.boxes[i].value()






#Created on 17 Aug 2014

#@author: neil.butcher


from .models import Measurement as MeasurementObject, UnitMeasurementException
from .models import Unit as UnitObject
from .ConglomerateWidgets import UnitEntryField, SingleMeasurementEntryFieldStack, MeasurementEntryGridField
from .MeasurementWidgets import UnitDisplay, UnitComboBox, UnitSpinBox
from .UnitComboDelegate import UnitComboDelegate
from .CurrentUnitSetter import setter as _setter
from . import SelectionMenu

_measurements = {}

def menu(parent, measurements=None):
    if measurements is None:
        #only those measurements which have been used for something
        return SelectionMenu.menu(_measurements.values(),parent)
    else:
        return SelectionMenu.menu([m for m in measurements if m in _measurements.values()], parent)

def Measurement(name):
    """
    :type name: str
    :rtype: MeasurementObject
    :raises UnitMeasurementException
    """
    if name not in _measurements:
        _measurements[name] = MeasurementObject(name)
    return _measurements[name]


def Unit(measurement, name):
    """
    :type measurement: MeasurementObject
    :type name: str
    :rtype: UnitObject
    :raises UnitMeasurementException
    """
    for u in measurement.units:
        if u.name == name:
            return u
    for u in measurement.units:
        if name in u.alias():
            return u
    raise UnitMeasurementException("There is no unit for " + str(measurement) + " by the name: " + name)

def changedSignal():
    """
    :rtype: callable
    """
    return _setter.changed

def measurements_list():
    return('Length','Angle','Pressure','Temperature','Volume', 'Mass', 'Proportion', 'Density', 'Force', 'Power', 'Time', 'AngularSpeed', 'Torque', 'None', 'Speed')



#Created on 14 Aug 2014

#@author: neil.butcher


from PyQt4 import QtCore, QtGui


class UnitComboDelegate(QtGui.QStyledItemDelegate):
    
    def __init__(self, parent):
        QtGui.QStyledItemDelegate.__init__(self, parent)
        self.parent = parent

    def createEditor(self, parent, option, index):
    
        measurement = index.data(QtCore.Qt.UserRole)
        self.itemslist = measurement.units
        names_list = []
        for i in self.itemslist:
            names_list.append(i.name)
        self.editor = QtGui.QComboBox(parent)
        self.editor.addItems(names_list)
        self.editor.setCurrentIndex(0)
        self.editor.installEventFilter(self)    
        return self.editor

    def setEditorData(self, editor, index):
        text = index.data(QtCore.Qt.DisplayRole)
        pos = self.editor.findText(text)
        if pos == -1:  
            pos = 0
        self.editor.setCurrentIndex(pos)
    
    def setModelData(self, editor, model, index):
        text = self.editor.currentText()
        i = self.editor.currentIndex()
        unit = self.itemslist[i]
        model.setData(index, unit)
    
    def updateEditorGeometry(self, editor, option, index):
        r = option.rect
        r.setSize(editor.sizeHint())
        self.editor.setGeometry(r)



#Created on 14 Apr 2016

#@author: neil.butcher


import sys

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import pyqtSignal



def menu(measurements_list, parent):
    menu = QtGui.QMenu('Units', parent)

    for m in measurements_list:
        sm = _menu_for_measurement(m,menu)
        menu.addMenu(sm)

    return menu

def _menu_for_measurement(a_measurement, parent):
    m = QtGui.QMenu(a_measurement.name, parent)
    g = QtGui.QActionGroup(m)

    for u in a_measurement.units:
        a = _action_for_unit(u, m)
        g.addAction(a)
        m.addAction(a)

    return m


def _action_for_unit(a_unit, parent):

    action = QtGui.QAction(a_unit.name, parent)
    action.triggered.connect(a_unit.becomeCurrentNormalUnit)
    action.setCheckable(True)
    action.setChecked(a_unit is a_unit.currentUnit())
    return action



#Created on 14 Aug 2014

#@author: neil.butcher


from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import pyqtSignal

from .CurrentUnitSetter import setter




class UnitDisplay(QtGui.QWidget):
    def __init__(self, parent, measurement=None, measurementLabel='normal'):
        QtGui.QWidget.__init__(self, parent)
        self.layout = QtGui.QVBoxLayout(self)
        self._label = QtGui.QLabel('', self)
        self.layout.addWidget(self._label)
        self.layout.setMargin(2)
        self.measurement = measurement
        self._measurementLabel = measurementLabel
        setter.changed.connect(self.currentUnitChangedElsewhere)
        self._update()

    @QtCore.pyqtSlot(str, str, str)
    def currentUnitChangedElsewhere(self, measName, unitName, measurementLabel):
        if self.measurement == None:
            pass
        elif not measName == self.measurement.name:
            pass
        elif not measurementLabel == self._measurementLabel:
            pass
        else:
            self._updateText(unitName)

    def setMeasurement(self, measurement):
        self.measurement = measurement
        self._update()

    def setMargin(self, margin):
        self.layout.setMargin(margin)

    def _update(self):
        if self.measurement == None:
            self._updateText('')
        else:
            self._updateText(self.measurement.currentUnit(label=self._measurementLabel).name)

    def _updateText(self, txt):
        self._label.setText(txt)


class UnitComboBox(QtGui.QWidget):
    def __init__(self, parent, measurement=None, measurementLabel='normal'):
        QtGui.QWidget.__init__(self, parent)
        self.layout = QtGui.QVBoxLayout(self)
        self.layout.setMargin(2)
        self._box = QtGui.QComboBox(self)
        self.layout.addWidget(self._box)
        self._measurementLabel = measurementLabel
        self._box.currentIndexChanged.connect(self.changedToIndex)
        setter.changed.connect(self.currentUnitChangedElsewhere)
        self.setMeasurement(measurement)
        self._update()

    @QtCore.pyqtSlot(str, str, str)
    def currentUnitChangedElsewhere(self, measName, unitName, measurementLabel):
        if self.measurement == None:
            pass
        elif not measName == self.measurement.name:
            pass
        elif not measurementLabel == self._measurementLabel:
            pass
        else:
            self._update()

    def setMeasurement(self, measurement):
        self.measurement = None
        self._box.clear()
        if measurement is None:
            pass
        else:
            self.itemslist = measurement.units
            namesList = []
            for i in self.itemslist:
                namesList.append(i.name)
            self._box.addItems(namesList)
        self.measurement = measurement
        self._update()

    def setMargin(self, margin):
        self.layout.setMargin(margin)

    def _update(self):
        if self.measurement is None:
            pass
        else:
            text = self.measurement.currentUnit(label=self._measurementLabel).name
            pos = self._box.findText(text)
            if pos == -1:
                pos = 0
            self._box.setCurrentIndex(pos)

    @QtCore.pyqtSlot(int)
    def changedToIndex(self, i):
        if not self.measurement == None:
            unit = self.itemslist[i]
            setter.setMeasurementUnit(self.measurement, unit, self._measurementLabel)


class AddaptiveDoubleSpinBox(QtGui.QDoubleSpinBox):
    def textFromValue(self, value):
        s = '{0:g}'.format(value)
        return s


class UnitSpinBox(QtGui.QWidget):
    valueChanged = pyqtSignal(float)
    editingFinished = pyqtSignal()

    def __init__(self, parent, measurement=None, delta=False, measurementLabel='normal'):
        QtGui.QWidget.__init__(self, parent)
        self.layout = QtGui.QVBoxLayout(self)
        self.layout.setMargin(2)
        self._box = AddaptiveDoubleSpinBox(self)
        self._box.setButtonSymbols(QtGui.QAbstractSpinBox.NoButtons)
        self._box.setMaximum(2.0e30)
        self._box.setMinimum(-2.0e30)
        self._box.setDecimals(12)
        self.layout.addWidget(self._box)
        self._box.valueChanged.connect(self._valueChanged)
        self._box.editingFinished.connect(self._editingFinished)
        setter.changed.connect(self.currentUnitChangedElsewhere)
        self.delta = delta
        self._baseValue = None
        self._measurementLabel = measurementLabel
        self.setMeasurement(measurement)
        self._update()

    @QtCore.pyqtSlot(str, str, str)
    def currentUnitChangedElsewhere(self, measName, unitName, measurementLabel):
        if self.measurement is None:
            pass
        elif not measName == self.measurement.name:
            pass
        elif not measurementLabel == self._measurementLabel:
            pass
        else:
            self._update()

    def setMeasurement(self, measurement):
        self.measurement = measurement
        self._update()

    def setMargin(self, margin):
        self.layout.setMargin(margin)

    def unit(self):
        return self.measurement.currentUnit(label=self._measurementLabel)

    def _update(self):
        if self._baseValue is None:
            self._box.clear()
        elif self.measurement is None:
            self._box.setValue(self._baseValue)
        elif self.delta:
            scaledValue = self.unit().scaledDeltaValueOf(self._baseValue)
            self._box.setValue(scaledValue)
        else:
            scaledValue = self.unit().scaledValueOf(self._baseValue)
            self._box.setValue(scaledValue)

    def setValue(self, baseValue):
        self._baseValue = baseValue
        self._update()

    def _valueChanged(self, scaledValue):
        if scaledValue is None:
            newValue = None
        elif self.measurement is None:
            newValue = scaledValue
        elif self.delta:
            newValue = self.unit().baseDeltaValueFrom(scaledValue)
        else:
            newValue = self.unit().baseValueFrom(scaledValue)

        a = self._baseValue
        b = newValue
        if a is None or abs(a - b) > max(abs(a), abs(b)) * 1e-8:
            self._baseValue = newValue
            self.valueChanged.emit(self._baseValue)

    def _editingFinished(self):
        self.editingFinished.emit()

    def value(self):
        return self._baseValue



#Created on 12 Aug 2014

#@author: neil.butcher

import os
from shutil import copyfile
root_filename =  os.path.join(os.path.dirname(__file__) , 'measurements' ,'measurements_root.db')

filename = os.path.join(os.path.dirname(__file__) , 'measurements' , 'measurements_temp.db')

if not os.path.isfile(filename):
    copyfile(root_filename,filename)




from setuptools import setup, find_packages
import os

setup(
    name='pyqt-units',
    packages=find_packages(exclude=[]),
    version='2.0',
    description='System of units for PyQt',
    author='Neil Butcher',
    url='https://github.com/ergoregion/pyqt-units.git',
    license='MIT',
    keywords=['units pyqt measurements'],
    install_requires=[],
    python_requires='>=3',
    package_data={
        'pyqt_units': [os.path.join('measurements','measurements_root.db')],
    },
    classifiers=[],
)




#Created on 14 Aug 2014

#@author: neil.butcher


from PyQt4 import QtCore
from PyQt4.QtCore import pyqtSignal


class CurrentUnitSetter(QtCore.QObject):
    
    changed = pyqtSignal(str,str,str)
    
    def setMeasurementUnit(self, m, u, label='normal'):
        """
        :type m: Measurement
        :type u: Unit
        :type label: str
        """
        m.setCurrentUnit(u, label)
        self.changed.emit(m.name, u.name, label)

setter = CurrentUnitSetter()  # Module wide item to transfer setting changes